<?php
/*
 * * ******************************************************************
 *
 *   CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *   AUTHOR                         ->     dariusz.bi@modulesgarden.com
 *   CONTACT                        ->       contact@modulesgarden.com
 *
 *  This software is furnished under a license and may be used and copied
 *  only  in  accordance  with  the  terms  of such  license and with the
 *  inclusion of the above copyright notice.  This software  or any other
 *  copies thereof may not be provided or otherwise made available to any
 *  other person.  No title to and  ownership of the  software is  hereby
 *  transferred.
 *
 * * ******************************************************************
 */

class CertumSSLCertificate extends abstractModel
{
    public $userId;
    public $domainId;
    public $orderId;
    public $csr;
    public $key;
    public $crt;
    public $certOrgContacts;
    public $certContacts;
    public $certDetails;
    public $certConfigurations;
    public $orderVoucher;
    public $taxIdentificationNumber;
    public $verifiPhoneNumber;
    public $reissueEmail;
    public $errors  = array();
    public $details = array();

    function __construct()
    {

    }

    function getApproverList()
    {
        
    }

    function generateCSR($forReissue = false)
    {
        $params = $this->certOrgContacts;

        $params['domain'] = $this->domainId;

        if (!isset($params['address1'])) {
            $params['address1'] = $this->certContacts['address'];
        }

        if (isset($this->certDetails['organisation_unit']) && $this->certDetails['organisation_unit'] != '') {
            $params['organisation_unit'] = $this->certDetails['organisation_unit'];
        }
        if ($forReissue === true) {
            $certOrdersModel   = new MGCertOrders(array('_tableName' => 'certumssl_orders'));
            $params            = array();
            $params            = $certOrdersModel->getDetailsByOrderId($this->orderId);
            $params['reissue'] = true;
        }

        $genResult = CertumSSLDriver::localAPI()->generateCsr($params);

        if (isset($genResult['error'])) {
            $this->errors[] = $genResult['error'];
            return;
        }

        $this->csr     = $genResult['csr'];
        $this->key     = $genResult['key'];
        $this->crt     = $genResult['crt'];
        $this->details = $genResult['details'];
    }

    function placeCertRequest()
    {
        $certumSSLAccount = new CertumSSLAccount($this->userId);
        //Set connection by productCode from VoucherCode
        $productCode      = $certumSSLAccount->getProductCodeFromToken($this->orderVoucher);
        $certumSSLAccount->setConnectionParams($productCode);

        if (is_array($this->domainId)) {
            $FQDN = $this->domainId[0];
        } else {
            if (strpos($this->domainId, "*.") !== false) {
                $FQDN = str_replace("*.", "", $this->domainId);
            } else {
                $FQDN = $this->domainId;
            }
        }

        $params = array(
            'csr' => $this->csr,
            'organizationName' => isset($this->certDetails['organisation']) ? $this->certDetails['organisation'] : $this->certContacts["organisationName"],
            'cert_contactorg_name' => $this->certContacts['cert_contactorg_name'],
            'customer' => $this->userId,
            'productvoucherCode' => $this->orderVoucher,
            'requestorInfo' => $this->certContacts,
            'taxIdentificationNumber' => $this->certContacts["taxNumber"],
            'verificationPhoneNumber' => $this->certContacts["verificationPhoneNumber"],
            'FQDN' => $FQDN,
            'domains' => $this->domainId
        );

        if (isset($this->certDetails['email']) && $this->certDetails['email'] != "") {
            $params['email'] = $this->certDetails['email'];
        }

        $request = $certumSSLAccount->quickOrder($params);

        if (empty($certumSSLAccount->errors) && $request != false) {

            $orderID       = $request->orderID;
            $this->orderId = $orderID;
            $addedDate     = $request->responseHeader->timestamp;
            //Multidomain
            if (is_array($request->verifications->verification)) {
                foreach ($request->verifications->verification as $verification) {

                    $approveCode = $verification->code;
                    $domain      = $verification->fQDN;

                    if (is_array($domain)) {
                        $domain = implode(",", $domain);
                    }

                    $certOrderParams = array(
                        '_tableName' => 'certumssl_orders',
                        'relId' => $certumSSLAccount->relid,
                        'userId' => $this->userId,
                        'domainId' => str_replace("*.", "", $domain),
                        'orderId' => $orderID,
                        'csr' => $this->csr,
                        'key' => $this->key,
                        'crt' => $this->crt,
                        'details' => json_encode($this->details),
                        'approveCode' => $approveCode,
                        'orderVoucher' => $this->orderVoucher,
                        'activateDate' => date("Y-m-d H:i:s", strtotime($addedDate)),
                        'status' => 'PENDING'
                    );

                    $certyficateOrder = new MGCertOrders($certOrderParams);
                    //To do not add second veryfications
                    if ($FQDN == $verification->fQDN) {
                        $certyficateOrder->create();
                    }
                }

                return true;
            }
            //single Domain
            if (is_array($this->domainId) && count($this->domainId) > 1) {
                $domain = implode(",", $this->domainId);
            } elseif (is_array($this->domainId) && count($this->domainId) == 1) {
                $domain = $this->domainId[0];
            } else {
                $domain = $this->domainId;
            }

            $approveCode     = $request->verifications->verification->code;
            $certOrderParams = array(
                '_tableName' => 'certumssl_orders',
                'relId' => $certumSSLAccount->relid,
                'userId' => $this->userId,
                'domainId' => str_replace("*.", "", $domain),
                'orderId' => $orderID,
                'csr' => $this->csr,
                'key' => $this->key,
                'crt' => $this->crt,
                'details' => json_encode($this->details),
                'approveCode' => $approveCode,
                'orderVoucher' => $this->orderVoucher,
                'activateDate' => date("Y-m-d H:i:s", strtotime($addedDate)),
                'status' => 'PENDING'
            );

            $certyficateOrder = new MGCertOrders($certOrderParams);
            $certyficateOrder->create();
            return true;
        }

        $this->errors = $certumSSLAccount->errors;
    }

    function verifyDomain($code, $relid)
    {
        $certumSSLAccount = new CertumSSLAccount($this->userId);
        $certumSSLAccount->setConnectionParams(null, $relid);
        $params           = array('code' => $code);
        $request          = $certumSSLAccount->verifyDomain($params);

        if (empty($certumSSLAccount->errors) && $request != false) {
            return true;
        } elseif ($certumSSLAccount->errors["verifyDomain"][0][0]['number'] == 1185) {
            return "exist";
        }
    }

    function checkVoucher()
    {
        //should return status example: available , reedemed , notexist etc 
        //For now we set fake response 
        return 'available';
    }

    function getVoucherDVMethod()
    {
        
    }

    function getVoucherParams()
    {
        
    }

    function setVoucherProductCode()
    {

    }

    function checkDomainStatus()
    {
        
    }

    function getOrderDetails($relid = null)
    {
        $certumSSLAccount = new CertumSSLAccount($this->userId);

        if (isset($relid)) {
            $certumSSLAccount->setConnectionParams(null, $relid);
        } else {
            $certumSSLAccount->setConnectionParams();
        }

        $params          = array('orderid' => $this->orderId);
        $params['relid'] = $relid;
        $request         = $certumSSLAccount->orderDetails($params);

        if (empty($certumSSLAccount->errors) && $request != false) {
            return $request;
        }

        $this->errors = $certumSSLAccount->errors;
        return false;
    }

    function setOrderStatus($status)
    {
        $certOrderParams = array(
            '_tableName' => 'certumssl_orders',
            'userId' => $this->userId,
            'domainId' => $this->domainId,
            'orderId' => $this->orderId,
            'status' => $status
        );

        $certyficateOrder = new MGCertOrders($certOrderParams);
        $certyficateOrder->updateStatus();
    }

    function reissueCertyficate($serialNumber, $relid)
    {
        $certumSSLAccount = new CertumSSLAccount($this->userId);
        $certumSSLAccount->setConnectionParams(null, $relid);
        $params           = array(
            'serialNumber' => $serialNumber,
            'csr' => $this->csr
        );

        $request = $certumSSLAccount->reissueCertificate($params);

        if (empty($certumSSLAccount->errors) && !is_array($request) && $request != false) {

            $orderID                = $request->orderID;
            // Musimy podmienić stary order id na nowy
            $certReissueOrderParams = array(
                '_tableName' => 'certumssl_reissued_orders',
                'orderId' => $orderID,
                'csr' => $this->csr,
                'private_key' => $this->key,
                'parentId' => $this->orderId
            );

            $certyficateReissueOrder = new MGReissuedOrders($certReissueOrderParams);
            $certyficateReissueOrder->create();
            return true;
        } elseif (is_array($request)) {
            if (isset($request['errors'])) {
                $this->errors = $request['errors'];
            }
        } else {
            $this->errors = $certumSSLAccount->errors;
        }
    }

    function revokeCertOrder($certSerialnumber, $relid)
    {
        $certumSSLAccount = new CertumSSLAccount($this->userId);
        $certumSSLAccount->setConnectionParams(null, $relid);
        $params           = array('serialNumber' => $certSerialnumber);
        $request          = $certumSSLAccount->revokeCertificate($params);

        if (empty($certumSSLAccount->errors) && $request != false) {
            return $request;
        }

        $this->errors = $certumSSLAccount->errors;
        return false;
    }

    function cancelCertOrder($relid)
    {
        $certumSSLAccount = new CertumSSLAccount($this->userId);
        $certumSSLAccount->setConnectionParams(null, $relid);
        $params           = array(
            'orderId' => $this->orderId
        );
        $request          = $certumSSLAccount->cancelOrder($params);

        if (empty($certumSSLAccount->errors) && $request != false) {
            return true;
        }

        $this->errors = $certumSSLAccount->errors;
        return false;
    }

    function reissueCertOrder()
    {
        
    }

    function checkVoucherDV()
    {

    }

    public static function getCountries()
    {
        return array(
            '' => 'Select',
            'AF' => 'Afghanistan',
            'AX' => 'Aland Islands',
            'AL' => 'Albania',
            'DZ' => 'Algeria',
            'AS' => 'American Samoa',
            'AD' => 'Andorra',
            'AO' => 'Angola',
            'AI' => 'Anguilla',
            'AQ' => 'Antarctica',
            'AG' => 'Antigua And Barbuda',
            'AR' => 'Argentina',
            'AM' => 'Armenia',
            'AW' => 'Aruba',
            'AU' => 'Australia',
            'AT' => 'Austria',
            'AZ' => 'Azerbaijan',
            'BS' => 'Bahamas',
            'BH' => 'Bahrain',
            'BD' => 'Bangladesh',
            'BB' => 'Barbados',
            'BY' => 'Belarus',
            'BE' => 'Belgium',
            'BZ' => 'Belize',
            'BJ' => 'Benin',
            'BM' => 'Bermuda',
            'BT' => 'Bhutan',
            'BO' => 'Bolivia',
            'BA' => 'Bosnia And Herzegovina',
            'BW' => 'Botswana',
            'BV' => 'Bouvet Island',
            'BR' => 'Brazil',
            'IO' => 'British Indian Ocean Territory',
            'BN' => 'Brunei Darussalam',
            'BG' => 'Bulgaria',
            'BF' => 'Burkina Faso',
            'BI' => 'Burundi',
            'KH' => 'Cambodia',
            'CM' => 'Cameroon',
            'CA' => 'Canada',
            'CV' => 'Cape Verde',
            'KY' => 'Cayman Islands',
            'CF' => 'Central African Republic',
            'TD' => 'Chad',
            'CL' => 'Chile',
            'CN' => 'China',
            'CX' => 'Christmas Island',
            'CC' => 'Cocos (Keeling) Islands',
            'CO' => 'Colombia',
            'KM' => 'Comoros',
            'CG' => 'Congo',
            'CD' => 'Congo, Democratic Republic',
            'CK' => 'Cook Islands',
            'CR' => 'Costa Rica',
            'CI' => 'Cote D\'Ivoire',
            'HR' => 'Croatia',
            'CU' => 'Cuba',
            'CY' => 'Cyprus',
            'CZ' => 'Czech Republic',
            'DK' => 'Denmark',
            'DJ' => 'Djibouti',
            'DM' => 'Dominica',
            'DO' => 'Dominican Republic',
            'EC' => 'Ecuador',
            'EG' => 'Egypt',
            'SV' => 'El Salvador',
            'GQ' => 'Equatorial Guinea',
            'ER' => 'Eritrea',
            'EE' => 'Estonia',
            'ET' => 'Ethiopia',
            'FK' => 'Falkland Islands (Malvinas)',
            'FO' => 'Faroe Islands',
            'FJ' => 'Fiji',
            'FI' => 'Finland',
            'FR' => 'France',
            'GF' => 'French Guiana',
            'PF' => 'French Polynesia',
            'TF' => 'French Southern Territories',
            'GA' => 'Gabon',
            'GM' => 'Gambia',
            'GE' => 'Georgia',
            'DE' => 'Germany',
            'GH' => 'Ghana',
            'GI' => 'Gibraltar',
            'GR' => 'Greece',
            'GL' => 'Greenland',
            'GD' => 'Grenada',
            'GP' => 'Guadeloupe',
            'GU' => 'Guam',
            'GT' => 'Guatemala',
            'GG' => 'Guernsey',
            'GN' => 'Guinea',
            'GW' => 'Guinea-Bissau',
            'GY' => 'Guyana',
            'HT' => 'Haiti',
            'HM' => 'Heard Island & Mcdonald Islands',
            'VA' => 'Holy See (Vatican City State)',
            'HN' => 'Honduras',
            'HK' => 'Hong Kong',
            'HU' => 'Hungary',
            'IS' => 'Iceland',
            'IN' => 'India',
            'ID' => 'Indonesia',
            'IR' => 'Iran, Islamic Republic Of',
            'IQ' => 'Iraq',
            'IE' => 'Ireland',
            'IM' => 'Isle Of Man',
            'IL' => 'Israel',
            'IT' => 'Italy',
            'JM' => 'Jamaica',
            'JP' => 'Japan',
            'JE' => 'Jersey',
            'JO' => 'Jordan',
            'KZ' => 'Kazakhstan',
            'KE' => 'Kenya',
            'KI' => 'Kiribati',
            'KR' => 'Korea',
            'KW' => 'Kuwait',
            'KG' => 'Kyrgyzstan',
            'LA' => 'Lao People\'s Democratic Republic',
            'LV' => 'Latvia',
            'LB' => 'Lebanon',
            'LS' => 'Lesotho',
            'LR' => 'Liberia',
            'LY' => 'Libyan Arab Jamahiriya',
            'LI' => 'Liechtenstein',
            'LT' => 'Lithuania',
            'LU' => 'Luxembourg',
            'MO' => 'Macao',
            'MK' => 'Macedonia',
            'MG' => 'Madagascar',
            'MW' => 'Malawi',
            'MY' => 'Malaysia',
            'MV' => 'Maldives',
            'ML' => 'Mali',
            'MT' => 'Malta',
            'MH' => 'Marshall Islands',
            'MQ' => 'Martinique',
            'MR' => 'Mauritania',
            'MU' => 'Mauritius',
            'YT' => 'Mayotte',
            'MX' => 'Mexico',
            'FM' => 'Micronesia, Federated States Of',
            'MD' => 'Moldova',
            'MC' => 'Monaco',
            'MN' => 'Mongolia',
            'ME' => 'Montenegro',
            'MS' => 'Montserrat',
            'MA' => 'Morocco',
            'MZ' => 'Mozambique',
            'MM' => 'Myanmar',
            'NA' => 'Namibia',
            'NR' => 'Nauru',
            'NP' => 'Nepal',
            'NL' => 'Netherlands',
            'AN' => 'Netherlands Antilles',
            'NC' => 'New Caledonia',
            'NZ' => 'New Zealand',
            'NI' => 'Nicaragua',
            'NE' => 'Niger',
            'NG' => 'Nigeria',
            'NU' => 'Niue',
            'NF' => 'Norfolk Island',
            'MP' => 'Northern Mariana Islands',
            'NO' => 'Norway',
            'OM' => 'Oman',
            'PK' => 'Pakistan',
            'PW' => 'Palau',
            'PS' => 'Palestinian Territory, Occupied',
            'PA' => 'Panama',
            'PG' => 'Papua New Guinea',
            'PY' => 'Paraguay',
            'PE' => 'Peru',
            'PH' => 'Philippines',
            'PN' => 'Pitcairn',
            'PL' => 'Poland',
            'PT' => 'Portugal',
            'PR' => 'Puerto Rico',
            'QA' => 'Qatar',
            'RE' => 'Reunion',
            'RO' => 'Romania',
            'RU' => 'Russian Federation',
            'RW' => 'Rwanda',
            'BL' => 'Saint Barthelemy',
            'SH' => 'Saint Helena',
            'KN' => 'Saint Kitts And Nevis',
            'LC' => 'Saint Lucia',
            'MF' => 'Saint Martin',
            'PM' => 'Saint Pierre And Miquelon',
            'VC' => 'Saint Vincent And Grenadines',
            'WS' => 'Samoa',
            'SM' => 'San Marino',
            'ST' => 'Sao Tome And Principe',
            'SA' => 'Saudi Arabia',
            'SN' => 'Senegal',
            'RS' => 'Serbia',
            'SC' => 'Seychelles',
            'SL' => 'Sierra Leone',
            'SG' => 'Singapore',
            'SK' => 'Slovakia',
            'SI' => 'Slovenia',
            'SB' => 'Solomon Islands',
            'SO' => 'Somalia',
            'ZA' => 'South Africa',
            'GS' => 'South Georgia And Sandwich Isl.',
            'ES' => 'Spain',
            'LK' => 'Sri Lanka',
            'SD' => 'Sudan',
            'SR' => 'Suriname',
            'SJ' => 'Svalbard And Jan Mayen',
            'SZ' => 'Swaziland',
            'SE' => 'Sweden',
            'CH' => 'Switzerland',
            'SY' => 'Syrian Arab Republic',
            'TW' => 'Taiwan',
            'TJ' => 'Tajikistan',
            'TZ' => 'Tanzania',
            'TH' => 'Thailand',
            'TL' => 'Timor-Leste',
            'TG' => 'Togo',
            'TK' => 'Tokelau',
            'TO' => 'Tonga',
            'TT' => 'Trinidad And Tobago',
            'TN' => 'Tunisia',
            'TR' => 'Turkey',
            'TM' => 'Turkmenistan',
            'TC' => 'Turks And Caicos Islands',
            'TV' => 'Tuvalu',
            'UG' => 'Uganda',
            'UA' => 'Ukraine',
            'AE' => 'United Arab Emirates',
            'GB' => 'United Kingdom',
            'US' => 'United States',
            'UM' => 'United States Outlying Islands',
            'UY' => 'Uruguay',
            'UZ' => 'Uzbekistan',
            'VU' => 'Vanuatu',
            'VE' => 'Venezuela',
            'VN' => 'Viet Nam',
            'VG' => 'Virgin Islands, British',
            'VI' => 'Virgin Islands, U.S.',
            'WF' => 'Wallis And Futuna',
            'EH' => 'Western Sahara',
            'YE' => 'Yemen',
            'ZM' => 'Zambia',
            'ZW' => 'Zimbabwe',
        );
    }

    function parseCertyficateData($certType = null)
    {

        $requireOrgFields = array(
            'OrganizationName' => 'name',
            'country' => 'address_country',
            'city' => 'address_city',
            'state' => 'address_region',
            'email' => 'address_email'
        );

        $requireCertDetailsFields = array();

        if ($certType != null) {
            $SSLProducts    = new SSLProducts();
            $productsFields = $SSLProducts->productsFields;
            $productFields  = $productsFields[$certType];
            $fields         = $SSLProducts->fields;

            if (!empty($productFields)) {
                foreach ($productFields as $productFieldShortcut => $productFieldStatus) {
                    $fieldParams = $fields[$productFieldShortcut];
                    if ($productFieldStatus == 'required') {
                        $requireCertDetailsFields[] = $fieldParams['name'];
                    }
                }
            }
        }

        $requireContactFields = array(
            'firstName' => 'firstname',
            'lastName' => 'lastname',
            'address' => 'address',
            'city' => 'city',
            'country' => 'country',
            'email' => 'email',
            'phone' => 'phone',
            'postalCode' => 'postalcode'
        );

        $errors = array();

        if (!empty($requireCertDetailsFields)) {
            foreach ($requireCertDetailsFields as $fieldName) {
                if (empty($this->certDetails[$fieldName]) || trim($this->certDetails[$fieldName]) == '') {
                    $errors['cert_certdetails_'.$fieldName] = true;
                }
            }
        }

        foreach ($requireOrgFields as $paramName => $fieldName) {
            if (empty($this->certOrgContacts[$paramName]) || trim($this->certOrgContacts[$paramName]) == '') {
                $errors['cert_org_'.$fieldName] = true;
            }
        }

        //Addictional Validation Documentation Quiq order pkt 4

        if (
            (isset($this->certOrgContacts['taxnumber']) && trim($this->certOrgContacts['taxnumber']) != '') &&
            (!isset($this->certOrgContacts['cert_contactorg_name']) || trim($this->certOrgContacts['cert_contactorg_name']) == '')
        ) {
            $errors['cert_contactorg_name'] = true;
        }

        if (
            (isset($this->certOrgContacts['cert_contactorg_name']) && trim($this->certOrgContacts['cert_contactorg_name']) != '') &&
            (!isset($this->certOrgContacts['taxnumber']) || trim($this->certOrgContacts['taxnumber'] == ''))
        ) {
            $errors['cert_contact_taxIdentificationNumber'] = true;
        }

        if ((isset($this->certOrgContacts['verificationPhoneNumber'])
            && trim($this->certOrgContacts['verificationPhoneNumber']) != '')
            && ((!isset($this->certOrgContacts['taxnumber']) || trim($this->certOrgContacts['taxnumber'] == '') || (!isset($this->certOrgContacts['cert_contactorg_name']) || trim($this->certOrgContacts['cert_contactorg_name'] == '')))  )
        ) {
            $errors['cert_org_verifiphonenumber'] = true;
        }
        //End Addictional validation

        if (!filter_var($this->certOrgContacts['email'], FILTER_VALIDATE_EMAIL)) {
            $errors['cert_org_address_email'] = true;
        }

        foreach ($requireContactFields as $paramName => $fieldName) {
            if (empty($this->certContacts[$paramName]) || trim($this->certContacts[$paramName]) == '') {
                $errors['cert_contact_'.$fieldName] = true;
            }
        }

        if (!filter_var($this->certContacts['email'], FILTER_VALIDATE_EMAIL)) {
            $errors['cert_contact_email'] = true;
        }

        return $errors;
    }

    function parseCertyficateReissueData()
    {
        $requireOrgFields = array(
            'OrganizationName' => 'name',
            'country' => 'address_country',
            'city' => 'address_city',
            'state' => 'address_region',
            'email' => 'address_email'
        );


        $errors = array();
        foreach ($requireOrgFields as $paramName => $fieldName) {
            if (empty($this->certOrgContacts[$paramName]) || trim($this->certOrgContacts[$paramName]) == '') {
                $errors['cert_org_'.$fieldName] = true;
            }
        }

        if (!filter_var($this->certOrgContacts['email'], FILTER_VALIDATE_EMAIL)) {
            $errors['cert_org_address_email'] = true;
        }

        return $errors;
    }

    function parseConfig()
    {
        
    }

    function parseContacts()
    {
        return false;
    }

    function parseReissueContacts()
    {

    }
}